@@ -31,8 +31,8 @@ class ChatGPT:
         moderation: bool = True,
         verbose: bool = False,
         window_size: tuple = (800, 600),
-        twocaptcha_apikey: str = '',
-        openai_auth_semi_automatic: bool = True,
+        captcha_solver: str = 'pypasser',
+        solver_apikey: str = '',
         login_cookies_path: str = '',
     ) -> None:
         '''
@@ -48,9 +48,9 @@ class ChatGPT:
         - moderation: (optional) Whether to enable message moderation. Default is `True`
         - verbose: (optional) Whether to print debug messages
         - window_size: (optional) window_size for web driver
-        - twocaptcha_apikey: (optional) 2captcha apikey, for solving reCAPTCHA. Use the apikey only for auth_type='openai'
-        - openai_auth_semi_automatic: (optional) allow solving reCAPTCHA by user when 2captcha method have failed.
-        - login_cookies_path: cookies path to be saved or loaded.
+        - captcha_solver: (optional) captcha solving method. Can be `pypasser` or `2captcha` or `manual`
+        - solver_apikey: (optional) captcha solver apikey, for solving reCAPTCHA. Use the apikey only for captcha_solver='2captcha'
+        - login_cookies_path: (optional) cookies path to be saved or loaded.
         '''
         self.__verbose = verbose
 
@@ -65,8 +65,8 @@ class ChatGPT:
         self.__auth_type = auth_type
         self.__window_size = window_size
         self.__moderation = moderation
-        self.__twocaptcha_apikey = twocaptcha_apikey
-        self.__openai_auth_semi_automatic = openai_auth_semi_automatic
+        self.__captcha_solver = captcha_solver
+        self.__solver_apikey = solver_apikey
         self.__login_cookies_path = login_cookies_path
         if self.__auth_type not in [None, 'google', 'windowslive', 'openai']:
             raise ValueError('Invalid authentication type')
@@ -77,6 +77,28 @@ class ChatGPT:
                 raise ValueError(
                     'Please provide either a session token or login credentials'
                 )
+            if self.__auth_type == 'openai':
+                if self.__captcha_solver not in ['pypasser', '2captcha', 'manual']:
+                    raise ValueError(
+                        'Invalid captcha solving method. Can be pypasser, 2captcha or manual'
+                    )
+                if self.__captcha_solver == '2captcha' and not self.__solver_apikey:
+                    raise ValueError('Please provide a 2captcha apikey')
+                if self.__captcha_solver == 'pypasser':
+                    try:
+                        import ffmpeg_downloader as ffdl
+                    except ModuleNotFoundError:
+                        raise ValueError(
+                            'Please install ffmpeg_downloader, PyPasser, and pocketsphinx by running `pip install ffmpeg_downloader PyPasser pocketsphinx`'
+                        )
+
+                    ffmpeg_installed = bool(ffdl.ffmpeg_version)
+                    self.__verbose_print('[0] ffmpeg installed:', ffmpeg_installed)
+                    if not ffmpeg_installed:
+                        import subprocess as sp
+
+                        sp.run(['ffdl', 'install'])
+                    os.environ['PATH'] += os.pathsep + ffdl.ffmpeg_dir
 
         self.__is_headless = (
             platform.system() == 'Linux' and 'DISPLAY' not in os.environ
@@ -175,17 +197,16 @@ class ChatGPT:
         self.driver.get('https://chat.openai.com/chat/' + self.conversation_id)
 
         # Dismiss the ChatGPT intro
-
         self.__check_and_dismiss_intro()
 
         # Check if there is an alert
-        self.__verbose_print('[init] Check if there is alert')
         self.__check_and_dismiss_alert()
 
     def __check_and_dismiss_alert(self):
-        alerts = self.__is_high_demand()
+        self.__verbose_print('[check_alert] Check if there is alert')
+        alerts = self.driver.find_elements(By.XPATH, '//div[@role="alert"]')
         if alerts:
-            self.__verbose_print('Dismissing alert')
+            self.__verbose_print('[check_alert] Dismissing alert')
             self.driver.execute_script(
                 """
             var element = document.querySelector('div[role="alert"]');
@@ -193,13 +214,16 @@ class ChatGPT:
                 element.parentNode.removeChild(element);
             """
             )
+        else:
+            self.__verbose_print('[check_alert] Did not found one')
 
     def __check_and_dismiss_intro(self):
+        self.__verbose_print('[check_intro] Check if there is intro')
         try:
             WebDriverWait(self.driver, 3).until(
                 EC.presence_of_element_located((By.ID, 'headlessui-portal-root'))
             )
-            self.__verbose_print('Dismissing intro')
+            self.__verbose_print('[check_intro] Dismissing intro')
             self.driver.execute_script(
                 """
             var element = document.getElementById('headlessui-portal-root');
@@ -208,28 +232,21 @@ class ChatGPT:
             """
             )
         except SeleniumExceptions.TimeoutException:
-            self.__verbose_print('[init] Did not found one')
+            self.__verbose_print('[check_intro] Did not found one')
             pass
 
-    def __is_high_demand(self) -> list or None:
-        '''
-        Check if there is an alert and close it
-        '''
-        alerts = self.driver.find_elements(By.XPATH, '//div[@role="alert"]')
-        return alerts
-
     def __save_chat_gpt_cookies(self, path):
         with open(path, 'w', encoding='utf-8') as f:
             cookies_list = self.driver.execute_cdp_cmd(
-                "Network.getCookies", {"urls": ["https://chat.openai.com/chat"]}
-            )["cookies"]
+                'Network.getCookies', {'urls': ['https://chat.openai.com/chat']}
+            )['cookies']
             json.dump(cookies_list, f, indent=2, ensure_ascii=False)
 
     def __load_chat_gpt_cookies(self, path):
         with open(path, 'r', encoding='utf-8') as f:
             cookies_list = json.load(f)
         for cookie in cookies_list:
-            if cookie["name"] == "__Secure-next-auth.session-token":
+            if cookie['name'] == '__Secure-next-auth.session-token':
                 self.driver.execute_cdp_cmd('Network.setCookie', cookie)
 
     def __login(self) -> None:
@@ -252,6 +269,9 @@ class ChatGPT:
                 (By.XPATH, '//div[text()="Welcome to ChatGPT"]')
             )
         )
+        WebDriverWait(self.driver, 5).until(
+            EC.element_to_be_clickable((By.XPATH, '//button[text()="Log in"]'))
+        )
         self.driver.find_element(By.XPATH, '//button[text()="Log in"]').click()
 
         WebDriverWait(self.driver, 5).until(
@@ -289,16 +309,16 @@ class ChatGPT:
     def __check_capacity(self, target_url):
         while True:
             try:
-                self.__verbose_print('Checking if ChatGPT is at capacity')
+                self.__verbose_print('[check_cap] Checking if ChatGPT is at capacity')
                 WebDriverWait(self.driver, 3).until(
                     EC.presence_of_element_located(
                         (By.XPATH, '//div[text()="ChatGPT is at capacity right now"]')
                     )
                 )
-                self.__verbose_print('ChatGPT is at capacity, retrying')
+                self.__verbose_print('[check_cap] ChatGPT is at capacity, retrying')
                 self.driver.get(target_url)
             except SeleniumExceptions.TimeoutException:
-                self.__verbose_print('ChatGPT is not at capacity')
+                self.__verbose_print('[check_cap] ChatGPT is not at capacity')
                 break
 
     def __google_login(self):
@@ -335,6 +355,7 @@ class ChatGPT:
             )
             self.__verbose_print('[login] Clicking Next')
             self.driver.find_element(By.XPATH, '//*[@id="passwordNext"]').click()
+
         # wait verification code
         try:
             self.__verbose_print('[login] Check if verification code is required')
@@ -382,10 +403,15 @@ class ChatGPT:
 
     def __2captcha_solve(self, enterprise=1, retry=8):
         self.driver.switch_to.default_content()
-        import twocaptcha
+        try:
+            import twocaptcha
+        except ModuleNotFoundError:
+            raise ValueError(
+                'twocaptcha module not found, please install it using `pip install 2captcha-python`'
+            )
 
         self.__verbose_print('[reCAPTCHA] trying twocaptcha max retry =', retry)
-        solver = twocaptcha.TwoCaptcha(self.__twocaptcha_apikey, pollingInterval=5)
+        solver = twocaptcha.TwoCaptcha(self.__solver_apikey, pollingInterval=5)
         # get result using 2Captcha
         el = self.driver.find_element(
             By.XPATH, '//div[@data-recaptcha-provider="recaptcha_enterprise"]'
@@ -401,7 +427,7 @@ class ChatGPT:
                     enterprise=enterprise,
                 )
             except Exception as e:
-                self.__verbose_print('twocaptcha solver error', e)
+                self.__verbose_print('[reCAPTCHA] twocaptcha solver error:', str(e))
             if result is not None:
                 break
         if result is None:
@@ -443,10 +469,11 @@ class ChatGPT:
                     )
                 )
             )
-            self.driver.find_element(
-                By.XPATH,
-                '//label[@class="rc-anchor-center-item rc-anchor-checkbox-label"]',
-            ).click()
+            # Create a div and block pypasser for some reasons
+            # self.driver.find_element(
+            #     By.XPATH,
+            #     '//label[@class="rc-anchor-center-item rc-anchor-checkbox-label"]',
+            # ).click()
             need_check_recaptcha_result = True
         except SeleniumExceptions.NoSuchFrameException as e:
             self.__verbose_print(e)
@@ -462,16 +489,25 @@ class ChatGPT:
                 )
             )
         except SeleniumExceptions.TimeoutException:
-            if self.__twocaptcha_apikey:
+            if self.__captcha_solver == '2captcha':
                 self.__2captcha_solve()
+            elif self.__captcha_solver == 'pypasser':
+                from pypasser import reCaptchaV2
+
+                self.__verbose_print('[reCAPTCHA] trying pypasser max retry = 3')
+                try:
+                    reCaptchaV2(self.driver)
+                except Exception as e:
+                    self.__verbose_print('[reCAPTCHA] pypasser solver error:', str(e))
 
         if need_check_recaptcha_result:
             if self.__have_recaptcha_value():
                 self.__verbose_print('[login] Congrats, solved reCAPTCHA.')
-            elif self.__openai_auth_semi_automatic:
+            else:
                 self.__verbose_print(
                     '[login] Ops, you have to solve reCAPTCHA on browser.'
                 )
+                self.driver.get(self.driver.current_url)
                 while need_check_recaptcha_result:
                     # check image selection reCAPTCHA
                     # self.__have_image_recaptcha()
@@ -630,37 +666,41 @@ class ChatGPT:
         '''
         Reset the conversation
         '''
-        self.__verbose_print('Resetting conversation')
+        chat_url = 'https://chat.openai.com/chat'
+        if not self.driver.current_url.startswith(chat_url):
+            return self.__verbose_print(f'[reset_convo] current_url is not {chat_url}.')
+
+        self.__verbose_print('[reset_convo] Resetting conversation')
         try:
             self.driver.find_element(By.LINK_TEXT, 'New chat').click()
         except SeleniumExceptions.NoSuchElementException:
+            self.__verbose_print('[reset_convo] New chat button not found')
             self.driver.save_screenshot('reset_conversation_failed.png')
 
     def clear_conversations(self) -> None:
+        '''
+        Clear all conversations
+        '''
         chat_url = 'https://chat.openai.com/chat'
-        if self.driver.current_url != chat_url:
-            self.__verbose_print(
-                '[Clearing conversations] current_url is not %s.' % chat_url
-            )
-            return
-        self.__verbose_print('[Clearing conversations] begin')
+        if not self.driver.current_url.startswith(chat_url):
+            return self.__verbose_print(f'[clear_convo] current_url is not {chat_url}.')
+
+        self.__verbose_print('[clear_convo] begin')
         try:
             self.driver.find_element(By.LINK_TEXT, 'Clear conversations').click()
         except SeleniumExceptions.NoSuchElementException:
             # the "Clear conversations" button does not show for the second time, maybe it is a bug.
-            self.__verbose_print(
-                '[Clearing conversations] Clear conversations button not found.'
-            )
+            self.__verbose_print('[clear_convo] Clear conversations button not found')
             pass
+
         try:
             self.driver.find_element(
                 By.LINK_TEXT, 'Confirm clear conversations'
             ).click()
         except SeleniumExceptions.NoSuchElementException:
-            self.__verbose_print(
-                '[Clearing conversations] Confirm clear conversations button not found.'
+            return self.__verbose_print(
+                '[clear_convo] Confirm clear conversations button not found'
             )
-            return
         try:
             WebDriverWait(self.driver, 20).until_not(
                 EC.presence_of_element_located(
@@ -670,16 +710,19 @@ class ChatGPT:
                     )
                 )
             )
-            self.__verbose_print('[Clearing conversations] successfully.')
+            self.__verbose_print('[clear_convo] Success')
         except SeleniumExceptions.TimeoutException:
-            self.__verbose_print('[Clearing conversations] failed.')
+            self.__verbose_print('[clear_convo] Failed')
 
     def refresh_chat_page(self) -> None:
+        '''
+        Refresh the chat page
+        '''
         chat_url = 'https://chat.openai.com/chat'
-        if self.driver.current_url == chat_url:
-            self.driver.get(chat_url)
-            self.__check_capacity(chat_url)
-            self.__check_and_dismiss_intro()
-            self.__check_and_dismiss_alert()
-        else:
-            self.__verbose_print('[refresh] current_url is not %s' % chat_url)
+        if not self.driver.current_url.startswith(chat_url):
+            return self.__verbose_print(f'[refresh] current_url is not {chat_url}')
+
+        self.driver.get(chat_url)
+        self.__check_capacity(chat_url)
+        self.__check_and_dismiss_intro()
+        self.__check_and_dismiss_alert()
